/*
 *	Dialog manager implementation
 *	issued: 2000/4/6
 *	author: c.mos@zob.ne.jp
 */

#include "dialog.h"

#define MBOX_BUTTON_MAX 4
#define CAPTION_CY 2
#define DIALOG_WIDTH_MAX (LCD_CHAR_CX - 4)

enum { MBOX_ICON, MBOX_TEXT, MBOX_CTR_COUNT };

enum { PROGBAR_L, PROGBAR_EMPTY, PROGBAR_HALF, PROGBAR_FULL, PROGBAR_R };


static Control ctr_buttons[MBOX_BUTTON_MAX];
static int button_count = 0;
static char *btn_labels[IDCOUNT] = { " Ok ", " Yes ", " No ", "Cancel" };
static Control ctr_mbox[MBOX_CTR_COUNT];
static Dialog dlg_mbox = { NULL, ctr_mbox, 0, MBOX_CTR_COUNT, -1 };

void draw_label(int x, int y, char *label, BOOL sel)
{
	set_color(sel ? CLR_HIGHLIGHT : CLR_NORMAL);
	text_put_string(x, y, label);
}

void draw_text(int x, int y0, int cx, int cy, char *text)
{
	if(text) {
		int y;
		char far *p = text;
		if(cy == 0) cy = 1;
		for(y = y0; y < y0 + cy; y++) {
			int width = 0;
			char far *p1 = p;
			if(*p1) {
				width = string_split(&p1, '\n');
				if(width > cx) {
					width = cx;
					p1 = p + string_skip(p, cx) - 1;
				}
				text_put_substring(x, y, p, width);
			}
			if(cx > width)
				text_fill_char(x + width, y, cx - width, ' ');
			p = p1;
			if(*p != '\0') p++;
		}
	}
}

void  draw_progress_bar(int x, int y, Control *ctr)
{
	int i;
	int n;
/*	if(frame) {
*/		put_color_font(x-1, y, (WORD far*)get_bmpres(BMP_CONTROLS) + (PROGBAR_L * 8));
		put_color_font(x + ctr->wnd.cx, y, (WORD far*)get_bmpres(BMP_CONTROLS) + (PROGBAR_R * 8));
		ctr->data.n = -1;
/*	}*/
	n = muldiv(ctr->wnd.cx, ctr->value * 2, ctr->range);
/*	if(n != ctr->data.n)*/ {
		int n2 = n >> 1;
		ctr->data.n = n;
		for_to(i, ctr->wnd.cx) {
			int font;
			if(i < n2) font = PROGBAR_FULL * 8;
			else if(i == n2 && (n & 1)) font = PROGBAR_HALF * 8;
			else font = PROGBAR_EMPTY * 8;
			put_color_font(x+i, y, (WORD far*)get_bmpres(BMP_CONTROLS) + font);
		}
	}
}

void draw_pushbutton(int x, int y, char *label, BOOL sel, BOOL frame)
{
	if(frame) {
		static Rect r;
		r.x = x; r.y = y;
		r.cx = string_width(label, 0);
		r.cy = 1;
		draw_button_frame(&r);
		inflate_rect(&r, 1, 1);
		set_palette_rect(&r, PAL_BTNFRAME);
		inflate_rect(&r, -1, -1);
		set_palette_rect(&r, PAL_WINDOW);
	}
	draw_label(x, y, label, sel);
}

void draw_control(Dialog *dlg, Control *ctr, int ctri, BOOL frame)
{
	int x = dlg->wnd.x + ctr->wnd.x;
	int y = dlg->wnd.y + ctr->wnd.y + CAPTION_CY;
	int cx = ctr->wnd.cx;
	char *datap = ctr->data.p;
	if(cx == 0)
		cx = dlg->wnd.cx - ctr->wnd.x;
	if(!(ctr->flags & CTF_HIDE)) {
		set_color(CLR_NORMAL);
		if(ctr->flags & CTF_OWNER_DRAW) {
			datap = ctr->data.fp(x, y, ctri - button_count, dlg->param);
			if(!datap) return;
		}
		switch(ctr->type) {
		case CTR_TEXT:
			draw_text(x, y, cx, ctr->wnd.cy, datap);
			break;
		case CTR_VALUE:
			text_put_numeric(x, y, ctr->wnd.cx, 0, ctr->data.n);
			break;
		case CTR_ICON: {
			WORD far *image = datap;
			if(ctr->data.n < ICON_COUNT) image = (WORD far*)get_bmpres(BMP_DLGICONS) + ((ctr->data.n) * (8 * 4));
			put_color_icon(x, y, 2, 2, image);
			break;
		}
		case CTR_PUSHBUTTON:
			draw_pushbutton(x, y, datap, dlg->ctrl_select == ctri, frame);
			break;
		case CTR_PROGRESSBAR:
			draw_progress_bar(x, y, ctr);
			break;
		case CTR_HLINE:
			draw_separator(x, y, cx, FRAME_SEP, PAL_BTNFRAME);
			break;
		}
	}
}

void draw_controls(Dialog *dlg)
{
	int i;
	for_to(i, dlg->ctrl_count) {
		draw_control(dlg, &dlg->controls[i], i + button_count, TRUE);
	}
	for_to(i, button_count) {
		draw_control(dlg, &ctr_buttons[i], i, TRUE);
	}
}

void redraw_dialog(Dialog *dlg, BOOL all)
{
	int i;
	for_to(i, dlg->ctrl_count) {
		Control* ctr = &dlg->controls[i];
		if(all || ctr->flags & CTF_REDRAW) {
			ctr->flags &= ~CTF_REDRAW;
			draw_control(dlg, ctr, i + button_count, FALSE);
		}
	}
}

void draw_caption(Dialog *dlg)
{
	int x = (dlg->wnd.cx - string_width(dlg->caption, 0)) >> 1;
	int y = dlg->wnd.y;
	set_color(CLR_CAPTION);
	text_fill_char(dlg->wnd.x, y, x, ' ');
	x += dlg->wnd.x;
	x += text_put_string(x, y, dlg->caption);
	text_fill_char(x, y, dlg->wnd.cx - x + dlg->wnd.x, ' ');
}

int init_buttons(WORD btn, int cx, int y)
{
	int i;
	int x = 1;
	btn &= MB_TYPEMASK;
	for_to(i, IDCOUNT) {
		if(btn & 1) {
			Control* ctr = &ctr_buttons[button_count];
			ctr->type = CTR_PUSHBUTTON;
			ctr->data.p = btn_labels[i];
			ctr->flags = 0;
			ctr->value = i;
			ctr->wnd.x = x;
			x += string_width(ctr->data.p, 0) + 1;
			ctr->wnd.y = y;
			button_count++;
		}
		btn >>= 1;
	}
	if(button_count) {
		if(x > cx) cx = x;
		else {
			x = ((cx - x) >> 1);
			for_to(i, button_count)
				ctr_buttons[i].wnd.x += x;
		}
	}
	return cx;
}

int do_buttons(Dialog* dlg)
{
	for(;;) {
		int select0 = dlg->ctrl_select;
		switch(key_wait()) {
		case KEY_RIGHT1:
			if(++dlg->ctrl_select >= button_count) dlg->ctrl_select = 0;
			goto Redraw;
		case KEY_LEFT1:
			if(dlg->ctrl_select) dlg->ctrl_select--;
			else dlg->ctrl_select = button_count - 1;
Redraw:
			draw_control(dlg, &ctr_buttons[select0], select0, FALSE);
			draw_control(dlg, &ctr_buttons[dlg->ctrl_select], dlg->ctrl_select, FALSE);
			break;
		case KEY_A:
			return ctr_buttons[dlg->ctrl_select].value;
		case KEY_B:
			return IDCANCEL;
		}
	}
}

void open_dialog(Dialog *dlg)
{
	if(button_count == 0 && dlg->type & MB_TYPEMASK)
		dlg->wnd.cx = init_buttons(dlg->type, dlg->wnd.cx, dlg->wnd.cy - (CAPTION_CY + 2));
	if(button_count == 1 && dlg->ctrl_select == 0)
		dlg->ctrl_select = -1;
	dlg->wnd.x = x_center(dlg->wnd.cx);
	dlg->wnd.y = y_center(dlg->wnd.cy);
	draw_frame(&dlg->wnd);
	fill_rect(&dlg->wnd, PAL_WINDOW, CLR_NORMAL);
	draw_caption(dlg);
	draw_controls(dlg);
}

void close_dialog(Dialog *dlg)
{
	button_count = 0;
	erase_frame(&dlg->wnd);
	draw_desktop(&dlg->wnd);
}

int message_box(char *text, char *caption, WORD type)
{
	int x = 1;
	int cy = 0;
	Size msg;
	Dialog* dlg = &dlg_mbox;
	Control* ctr;
	int id;
	int icon = (type & MB_ICONMASK) >> MB_ICONSHIFT;
	dlg->caption = caption;
	dlg->type = type;
	dlg->ctrl_select = (type & MB_DEFBUTTONMASK) >> MB_DEFBUTTONSHIFT;
	ctr = &ctr_mbox[MBOX_ICON];
	ctr->type = CTR_NULL;
	if(icon) {
		ctr->type = CTR_ICON;
		ctr->data.n = icon-1;
		ctr->wnd.x = x;
		ctr->wnd.y = 0;
		x += 3;
		cy = 2;
	}
	ctr = &ctr_mbox[MBOX_TEXT];
	ctr->type = CTR_TEXT;
	ctr->data.p = text;
	ctr->wnd.x = x;
	ctr->wnd.y = 0;
	string_size(text, &msg);
	ctr->wnd.cy = cy;
	cy = max(cy, msg.cy) + 1;
	dlg->wnd.cx = init_buttons(type, x + min(msg.cx, DIALOG_WIDTH_MAX), cy);
	dlg->wnd.cy = cy + CAPTION_CY + 2;
	open_dialog(dlg);
	id = do_buttons(dlg);
	close_dialog(dlg);
	return id;
}

