/*
 *	User Interface base definitions
 *	issued: 2000/4/2
 *	author: c.mos@zob.ne.jp
 */

#ifndef _GUI_H_
#define _GUI_H_

#include "std.h"

#define LCD_PIXEL_CX	224
#define LCD_PIXEL_CY	144
#define LCD_CHAR_CX		28
#define LCD_CHAR_CY		18
#define STATUS_Y (LCD_CHAR_CY - 1)

#define PALSFT 9
#define CFM_PALETTE	0x1E00

#define PAL_TILE 2
#define PAL_DESKTOP 4
#define PAL_WINDOW 3
#define PAL_FRAME 12
#define PAL_BTNFRAME 1

#define MESSAGE_MAX 80

#define x_center(cx) ((LCD_CHAR_CX - (cx)) >> 1)
#define y_center(cy) ((LCD_CHAR_CY - (cy)) >> 1)

typedef struct {
	int x;
	int y;
} Point;

typedef struct {
	int cx;
	int cy;
} Size;

typedef struct {
	int x;
	int y;
	int cx;
	int cy;
} Rect;

enum {
	FRAME_TOP_L, FRAME_TOP, FRAME_TOP_R,
	FRAME_L,	 FRAME_SEP, FRAME_R,
	FRAME_BTM_L, FRAME_BTM, FRAME_BTM_R, SPRITE_FONT_COUNT
};

enum {
	CLR_DESKTOP,
	CLR_NORMAL,
	CLR_HIGHLIGHT,
	CLR_GRAY,
	CLR_GRAY_HIGHLIGHT,
	CLR_STATUS,
	CLR_CAPTION,
	CLR_COUNT
};

enum {
	BMP_ICONS,
	BMP_DLGICONS,
	BMP_TILE,
	BMP_FRAME,
	BMP_CONTROLS
};

extern int colors[CLR_COUNT];

void set_color(int i);
void draw_desktop(Rect *r_clip);

void draw_frame(Rect *r);
void erase_frame(Rect *r);
void draw_button_frame(Rect *r);
void draw_separator(int x, int y, int width, int bmp, int pal);

void fill_rect(Rect *r, int pal, int col);
void reserve_sprite_font(void);
void set_palette_rect(Rect *r, int pal);
/*void put_mono_icon(int x, int y, int cx, int cy, BYTE *icon);*/
void put_color_icon(int x, int y, int cx, int cy, WORD far *icon);
void put_color_font(int x, int y, WORD far *font);
int string_width(char far *p, int len);
int string_size(char far *p, Size far *size);
int string_skip(char far *p, int width);
int string_split(char far * far *pp, int splitter);

BOOL intersect_rect(Rect *r, const Rect *r1, const Rect *r2);
void inflate_rect(Rect *r, int dx, int dy);

void far *get_bmpres(int n);
void font_set_far_colordata(unsigned char_no, unsigned char_count, void far *data_p);

#endif _GUI_H_
